# Copyright 2018 Rubrik, Inc.
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to
#  deal in the Software without restriction, including without limitation the
#  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
#  sell copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in
#  all copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#  DEALINGS IN THE SOFTWARE.

"""
This module contains the Rubrik SDK Physical class.
"""

from .api import Api
from .exceptions import InvalidParameterException, InvalidTypeException


class Physical(Api):
    """This class contains methods related to the managment of the Physical objects in the Rubrik cluster."""

    def add_physical_host(self, hostname, timeout=60):
        """Add a physical host to the Rubrik cluster.

        Arguments:
            hostname {str} or [list] -- The hostname(s) or IP Address(es) of the physical host you want to add to the Rubrik cluster.

        Keyword Arguments:
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {60})

        Returns:
            str -- No change required. The host '`hostname`' is already connected to the Rubrik cluster.
            dict -- The full API response for `POST /v1/host`.
        """

        if(len(hostname) == 0):
            raise InvalidParameterException("The provided hostname list is empty.")

        self.log('Searching the Rubrik cluster for the current hosts.')
        current_hosts = self.get('v1', '/host', timeout=timeout)

        if isinstance(hostname, list):

            for host in current_hosts['data']:
                for single_host in hostname:
                    if host['hostname'] == single_host:
                        hostname.remove(single_host)
                        self.log("The host '{}' is already connected to the Rubrik cluster. '{}' skipped.".format(
                            single_host, single_host))

            config = []

            self.log("Adding '{}' Physical Host(s)".format(len(hostname)))

            if len(hostname) != 0:
                for hosts in hostname:
                    config += [{
                        'hostname': hosts,
                        'hasAgent': True
                    }]

                self.log("Adding the following physical host(s): '{}'".format(hostname))
                return self.post('internal', '/host/bulk', config, timeout)
            else:
                return "No Change Required. All Hosts Already added or supplied list was empty"
        else:
            for host in current_hosts['data']:
                if host['hostname'] == hostname:
                    return "No change required. The host '{}' is already connected to the Rubrik cluster.".format(
                        hostname)

            config = {}
            config['hostname'] = hostname
            config['hasAgent'] = True

            self.log("Adding the host '{}' to the Rubrik cluster.".format(hostname))
            return self.post('v1', '/host', config, timeout)

    def delete_physical_host(self, hostname, timeout=120):
        """Delete a physical host from the Rubrik cluster.

        Arguments:
            hostname {str} -- The hostname or IP Address of the physical host you wish to remove from the Rubrik cluster.

        Keyword Arguments:
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {120})

        Returns:
            str -- No change required. The host '`hostname`' is not connected to the Rubrik cluster.
            dict -- The full API response for `DELETE /v1'/host/{host_id}`.
        """

        self.log('Searching the Rubrik cluster for the current hosts.')
        current_hosts = self.get('v1', '/host', timeout=timeout)

        host_present = False

        for host in current_hosts['data']:
            if host['hostname'] == hostname:
                host_present = True
                host_id = host['id']
                break

        if not host_present:
            return "No change required. The host '{}' is not connected to the Rubrik cluster.".format(
                hostname)

        self.log("Deleting the host '{}' from the Rubrik cluster.".format(hostname))
        return self.delete('v1', '/host/{}'.format(host_id), timeout=timeout)

    def create_physical_fileset(self, name, operating_system, include, exclude, exclude_exception, follow_network_shares=False, backup_hidden_folders=False, timeout=15):  # pylint: ignore
        """Create a Fileset for a Linux or Windows machine.

        Arguments:
            name {str} -- The name of the Fileset you wish to create.
            operating_system {str} -- The operating system type of the Fileset you are creating. (choices: {Linux, Windows.})
            include {list} -- The full paths or wildcards that define the objects to include in the Fileset backup (ex: ['/usr/local', '*.pdf']).
            exclude {list} -- The full paths or wildcards that define the objects to exclude from the Fileset backup (ex: ['/user/local/temp', '*.mov', '*.mp3']).
            exclude_exception {list} -- The full paths or wildcards that define the objects that are exempt from the `excludes` variables. (ex. ['/company/*.mp4').

        Keyword Arguments:
            follow_network_shares {bool} -- Include or exclude locally-mounted remote file systems from backups. (default: {False})
            backup_hidden_folders {bool} -- Include or exclude hidden folders inside locally-mounted remote file systems from backups. (default: {False})
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {15})

        Returns:
            str -- No change required. The Rubrik cluster already has a `operating_system` Fileset named '`name`' configured with the provided variables.
            dict -- The full response for the `POST /internal/fileset_template/bulk` API endpoint.
        """

        valid_operating_system = ['Linux', 'Windows']

        if operating_system not in valid_operating_system:
            raise InvalidParameterException("The create_physical_fileset() operating_system argument must be one of the following: {}.".format(
                valid_operating_system))

        if isinstance(follow_network_shares, bool) is False:
            raise InvalidTypeException("The 'follow_network_shares' argument must be True or False.")
        elif isinstance(backup_hidden_folders, bool) is False:
            raise InvalidTypeException("The 'backup_hidden_folders' argument must be True or False.")
        elif isinstance(include, list) is False:
            raise InvalidTypeException("The 'include' argument must be a list object.")
        elif isinstance(exclude, list) is False:
            raise InvalidTypeException("The 'exclude' argument must be a list object.")
        elif isinstance(exclude_exception, list) is False:
            raise InvalidTypeException("The 'exclude_exception' argument must be a list object.")

        config = {}
        config['name'] = name
        config['includes'] = sorted(include)
        config['excludes'] = sorted(exclude)
        config['exceptions'] = sorted(exclude_exception)
        config['allowBackupHiddenFoldersInNetworkMounts'] = backup_hidden_folders
        config['allowBackupNetworkMounts'] = follow_network_shares
        config['operatingSystemType'] = operating_system

        self.log("create_fileset: Searching the Rubrik cluster for all current {} Filesets.".format(operating_system))
        current_filesets = self.get(
            'v1', '/fileset_template?primary_cluster_id=local&operating_system_type={}&name={}'.format(
                operating_system, name), timeout=timeout)

        current_config = {}
        if current_filesets['data']:
            current_config['name'] = current_filesets['data'][0]['name']
            current_config['includes'] = sorted(current_filesets['data'][0]['includes'])
            current_config['excludes'] = sorted(current_filesets['data'][0]['excludes'])
            current_config['exceptions'] = sorted(current_filesets['data'][0]['exceptions'])
            current_config['allowBackupHiddenFoldersInNetworkMounts'] = current_filesets['data'][0]['allowBackupHiddenFoldersInNetworkMounts']
            current_config['operatingSystemType'] = current_filesets['data'][0]['operatingSystemType']
            current_config['allowBackupNetworkMounts'] = current_filesets['data'][0]['allowBackupNetworkMounts']

        if current_config == config:
            return "No change required. The Rubrik cluster already has a {} Fileset named '{}' configured with the provided variables.".format(
                operating_system, name)

        # Add the config dict to a list
        model = []
        model.append(config)

        self.log("create_fileset: Creating the '{}' Fileset.".format(name))
        return self.post('internal', '/fileset_template/bulk', model, timeout=timeout)

    def create_nas_fileset(self, name, share_type, include, exclude, exclude_exception, follow_network_shares=False, timeout=15):  # pylint: ignore
        """Create a NAS Fileset.

        Arguments:
            name {str} -- The name of the Fileset you wish to create.
            share_type {str} -- The type of NAS Share you wish to backup. (choices: {NFS, SMB})
            include {list} -- The full paths or wildcards that define the objects to include in the Fileset backup.
            exclude {list} -- The full paths or wildcards that define the objects to exclude from the Fileset backup.
            exclude_exception {list} -- The full paths or wildcards that define the objects that are exempt from the `excludes` variables.

        Keyword Arguments:
            follow_network_shares {bool} -- Include or exclude locally-mounted remote file systems from backups. (default: {False})
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {15})

        Returns:
            str -- No change required. The Rubrik cluster already has a NAS Fileset named '`name`' configured with the provided variables.
            dict -- The full response for the `POST /internal/fileset_template/bulk` API endpoint.
        """

        valid_share_type = ['NFS', 'SMB']

        if share_type not in valid_share_type:
            raise InvalidParameterException(
                "The create_fileset() share_type argument must be one of the following: {}.".format(valid_share_type))

        if isinstance(follow_network_shares, bool) is False:
            raise InvalidTypeException("The 'follow_network_shares' argument must be True or False.")
        elif isinstance(include, list) is False:
            raise InvalidTypeException("The 'include' argument must be a list object.")
        elif isinstance(exclude, list) is False:
            raise InvalidTypeException("The 'exclude' argument must be a list object.")
        elif isinstance(exclude_exception, list) is False:
            raise InvalidTypeException("The 'exclude_exception' argument must be a list object.")

        config = {}
        config['name'] = name
        config['includes'] = sorted(include)
        config['excludes'] = sorted(exclude)
        config['exceptions'] = sorted(exclude_exception)
        config['allowBackupHiddenFoldersInNetworkMounts'] = follow_network_shares
        config['shareType'] = share_type

        self.log("create_fileset: Searching the Rubrik cluster for all current NAS Filesets.")
        current_filesets = self.get(
            'v1', '/fileset_template?primary_cluster_id=local&operating_system_type=NONE&name={}'.format(name), timeout=timeout)

        current_config = {}
        if current_filesets['data']:
            current_config['name'] = current_filesets['data'][0]['name']
            current_config['includes'] = sorted(current_filesets['data'][0]['includes'])
            current_config['excludes'] = sorted(current_filesets['data'][0]['excludes'])
            current_config['exceptions'] = sorted(current_filesets['data'][0]['exceptions'])
            current_config['allowBackupHiddenFoldersInNetworkMounts'] = current_filesets['data'][0]['allowBackupHiddenFoldersInNetworkMounts']
            current_config['shareType'] = current_filesets['data'][0]['shareType']

        if current_config == config:
            return "No change required. The Rubrik cluster already has a NAS Fileset named '{}' configured with the provided variables.".format(
                name)

        # Add the config dict to a list
        model = []
        model.append(config)

        self.log("create_fileset: Creating the '{}' Fileset.".format(name))
        return self.post('internal', '/fileset_template/bulk', model, timeout=timeout)

    def assign_physical_host_fileset(self, hostname, fileset_name, operating_system, sla_name, include=None, exclude=None, exclude_exception=None, follow_network_shares=False, backup_hidden_folders=False, timeout=30):  # pylint: ignore
        """Assign a Fileset to a Linux or Windows machine. If you have multiple Filesets with identical names, you will need to populate the Filesets properties (i.e this functions keyword arguments)
        to find a specific match. Filesets with identical names and properties are not supported.

        Arguments:
            hostname {str} -- The hostname or IP Address of the physical host you wish to associate to the Fileset.
            fileset_name {str} -- The name of the Fileset you wish to assign to the Linux or Windows host.
            operating_system {str} -- The operating system of the physical host you are assigning a Fileset to. (choices: {Linux, Windows})
            sla_name {str} -- The name of the SLA Domain to associate with the Fileset.

        Keyword Arguments:
            include {list} -- The full paths or wildcards that define the objects to include in the Fileset backup (ex: ['/usr/local', '*.pdf']). (default: {None})
            exclude {list} -- The full paths or wildcards that define the objects to exclude from the Fileset backup (ex: ['/user/local/temp', '*.mov', '*.mp3']). (default: {None})
            exclude_exception {list} -- The full paths or wildcards that define the objects that are exempt from the `excludes` variables. (ex: ['/company/*.mp4']). (default: {None})
            follow_network_shares {bool} -- Include or exclude locally-mounted remote file systems from backups. (default: {False})
            backup_hidden_folders {bool} -- Include or exclude hidden folders inside locally-mounted remote file systems from backups. (default: {False})
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {30})

        Returns:
            str -- No change required. The `operating_system` Fileset '`fileset_name`' is already assigned to the SLA Domain '`sla_name`' on the physical host '`hostname`'.
            tuple -- When a new Fileset is created the following tuple will be returned: (Full API response from `POST /v1/fileset`, Full API response from `POST /v1/fileset/{id}`)
            dict -- When the Fileset already exsits but is assigned to the wrong the SLA the Full API response from `POST `v1/fileset/{id}` is returned.
        """

        valid_operating_system = ['Linux', 'Windows']

        if operating_system not in valid_operating_system:
            raise InvalidParameterException("The assign_physical_host_fileset() operating_system argument must be one of the following: {}.".format(
                valid_operating_system))

        if include is None:
            include = []

        if exclude is None:
            exclude = []

        if exclude_exception is None:
            exclude_exception = []

        if isinstance(follow_network_shares, bool) is False:
            raise InvalidTypeException("The 'follow_network_shares' argument must be True or False.")
        elif isinstance(backup_hidden_folders, bool) is False:
            raise InvalidTypeException("The 'backup_hidden_folders' argument must be True or False.")
        elif isinstance(include, list) is False:
            raise InvalidTypeException("The 'include' argument must be a list object.")
        elif isinstance(exclude, list) is False:
            raise InvalidTypeException("The 'exclude' argument must be a list object.")
        elif isinstance(exclude_exception, list) is False:
            raise InvalidTypeException("The 'exclude_exception' argument must be a list object.")

        self.log(
            "assign_physical_host_fileset: Searching the Rubrik cluster for the {} physical host {}.".format(
                operating_system,
                hostname))
        current_hosts = self.get('v1',
                                 '/host?operating_system_type={}&primary_cluster_id=local&hostname={}'.format(operating_system,
                                                                                                              hostname),
                                 timeout=timeout)

        if current_hosts['total'] >= 1:
            for host in current_hosts['data']:
                if host['hostname'] == hostname:
                    host_id = host['id']
                    break
        try:
            host_id
        except NameError:
            raise InvalidParameterException(
                "The Rubrik cluster is not connected to a {} physical host named '{}'.".format(operating_system, hostname))

        self.log("assign_physical_host_fileset: Searching the Rubrik cluster for all current {} Filesets.".format(operating_system))
        current_filesets_templates = self.get(
            'v1', '/fileset_template?primary_cluster_id=local&operating_system_type={}&name={}'.format(
                operating_system, fileset_name), timeout=timeout)

        number_of_matches = 0
        if current_filesets_templates['total'] == 0:
            raise InvalidParameterException(
                "The Rubrik cluster does not have a {} Fileset named '{}'.".format(
                    operating_system, fileset_name))
        elif current_filesets_templates['total'] > 1:
            for fileset_template in current_filesets_templates['data']:
                if fileset_template['name'] == fileset_name:
                    number_of_matches += 1

            if number_of_matches > 1:
                # If there are multiple Filesets with the same name us all of
                # the possible config values to try and find the correct
                # Fileset
                for fileset_template in current_filesets_templates['data']:
                    if fileset_template['name'] == fileset_name \
                            and fileset_template['includes'] == include \
                            and fileset_template['excludes'] == exclude \
                            and fileset_template['exceptions'] == exclude_exception \
                            and fileset_template['allowBackupHiddenFoldersInNetworkMounts'] == follow_network_shares \
                            and fileset_template['allowBackupNetworkMounts'] == backup_hidden_folders:
                        fileset_template_id = fileset_template['id']
                        number_of_matches = 1

                try:
                    fileset_template_id
                except NameError:
                    if number_of_matches > 0:
                        # If no unique matches are found provide an error
                        # message
                        if include != [] \
                                or exclude != [] \
                                or exclude_exception != [] \
                                or follow_network_shares\
                                or backup_hidden_folders:
                            # Error message that first checks to see if any of
                            # the extra variables are populated with anything
                            # besides the default (aka the user tried to be as
                            # unique as possible)
                            raise InvalidParameterException(
                                "The Rubrik cluster contains multiple {} Filesets named '{}' that match all of the populate function arguments. Please use a unique Fileset.".format(
                                    operating_system, fileset_name))
                        else:

                            raise InvalidParameterException(
                                "The Rubrik cluster contains multiple {} Filesets named '{}'. Please populate all function arguments to find a more specific match.".format(
                                    operating_system, fileset_name))

        if current_filesets_templates['total'] == 1 or number_of_matches == 1:
            for fileset_temmplate in current_filesets_templates['data']:
                if fileset_temmplate['name'] == fileset_name:
                    fileset_template_id = fileset_temmplate['id']

        self.log("assign_physical_host_fileset: Searching the Rubrik cluster for the SLA Domain '{}'.".format(sla_name))
        sla_id = self.object_id(sla_name, 'sla', timeout=timeout)

        self.log("assign_physical_host_fileset: Getting the properties of the {} Fileset.".format(fileset_name))
        current_fileset = self.get(
            'v1', '/fileset?primary_cluster_id=local&host_id={}&is_relic=false&template_id={}'.format(host_id, fileset_template_id), timeout=timeout)

        if current_fileset['total'] == 0:
            self.log(
                "assign_physical_host_fileset: Assigning the '{}' Fileset to the {} physical host '{}'.".format(
                    fileset_name,
                    operating_system,
                    hostname))

            config = {}
            config['hostId'] = host_id
            config['templateId'] = fileset_template_id

            create_fileset = self.post('v1', '/fileset', config, timeout)
            fileset_id = create_fileset['id']

            config = {}
            config['configuredSlaDomainId'] = sla_id
            assign_sla = self.patch('v1', '/fileset/{}'.format(fileset_id), config, timeout)

            return (create_fileset, assign_sla)
        elif current_fileset['total'] == 1 and current_fileset['data'][0]['configuredSlaDomainId'] != sla_id:

            self.log(
                "assign_physical_host_fileset: Assigning the '{}' SLA Domain to the '{}' Fileset attached to the {} physical host '{}'.".format(
                    sla_name,
                    fileset_name,
                    operating_system,
                    hostname))
            fileset_id = current_fileset['data'][0]['id']
            config = {}
            config['configuredSlaDomainId'] = sla_id
            return self.patch('v1', '/fileset/{}'.format(fileset_id), config, timeout)

        elif current_fileset['total'] == 1 and current_fileset['data'][0]['configuredSlaDomainId'] == sla_id:
            return "No change required. The {} Fileset '{}' is already assigned to the SLA Domain '{}' on the physical host '{}'.".format(
                operating_system, fileset_name, sla_name, hostname)



#! /usr/bin/env python

import setuptools

long_description = """
# Rubrik SDK for Python

This project provides a Python package that makes it easy to interact with the Rubrik CDM API.

The SDK has been tested against Python 2.7.6 and Python 3.6.4.

## Installation

Install from pip:

`pip install rubrik_cdm`

Install from source:
```
$ git clone https://github.com/rubrik-devops/rubrik-sdk-for-python
$ cd rubrik-sdk-for-python
$ python setup.py install
```
## Quick Start

* [Quick Start Guide](https://github.com/rubrikinc/rubrik-sdk-for-python/blob/master/docs/quick-start.md)

## Documentation

* [SDK Documentation](https://rubrik.gitbook.io/rubrik-sdk-for-python/)
* [Rubrik API Documentation](https://github.com/rubrikinc/api-documentation)

## Example

By default, the Rubrik SDK will attempt to read the the Rubrik Cluster credentials from the following environment variables:

* `rubrik_cdm_node_ip`
* `rubrik_cdm_username`
* `rubrik_cdm_password`

```py
import rubrik_cdm
rubrik = rubrik_cdm.Connect()
```

## Additional Links
* [VIDEO: Getting Start with the Rubrik SDK for Python](https://www.youtube.com/watch?v=TQM60X2_r_c&feature=youtu.be)
"""


setuptools.setup(
    name="rubrik_cdm",
    version="2.0.1",
    author="Rubrik Build",
    description="A Python package for interacting with the Rubrik CDM API.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/rubrik-devops/rubrik-sdk-for-python",
    packages=setuptools.find_packages(exclude=['tests']),
    classifiers=[
        "Intended Audience :: Information Technology",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.6"
    ],
    install_requires=[
        'requests >= 2.18.4',
        'python-dateutil',
        'pytz'
    ],
    tests_require=[
        'pytest'
    ],
    zip_safe=True,
)



# Copyright 2018 Rubrik, Inc.
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to
#  deal in the Software without restriction, including without limitation the
#  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
#  sell copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in
#  all copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#  DEALINGS IN THE SOFTWARE.

"""
This module contains the Rubrik SDK Cloud class.
"""

import os
import re
from .api import Api
from .exceptions import InvalidParameterException, CDMVersionException, InvalidTypeException


class Cloud(Api):
    """This class contains methods for the managment of Cloud related functionality on the Rubrik cluster."""

    def aws_s3_cloudout(self, aws_bucket_name, archive_name='default', aws_region=None, aws_access_key=None, aws_secret_key=None, kms_master_key_id=None, rsa_key=None, storage_class='standard', timeout=180):  # pylint: ignore
        """Add a new AWS S3 archival location to the Rubrik cluster.

        Arguments:
            aws_bucket_name {str} -- The name of the AWS S3 bucket you wish to use as an archive target. The bucket name will automatically have all whitespace removed, all letters lowercased, and can not contain any of the following characters: `_\/*?%.:\|<>`.

        Keyword Arguments:
            aws_region {str} -- The name of the AWS region where the bucket is located. If set to the default `None` keyword argument, we will look for a `AWS_DEFAULT_REGION` environment variable to pull the value from. (default: {None}) (choices: {ap-south-1, ap-northeast-2, ap-southeast-1, ap-southeast-2, ap-northeast-1, ca-central-1, cn-north-1, cn-northwest-1, eu-central-1, eu-west-1, eu-west-2, eu-west-3, sa-east-1, us-gov-west-1, us-west-1, us-east-1, us-east-2, us-west-2})
            aws_access_key {str} -- The access key of a AWS account with the required permissions. If set to the default `None` keyword argument, we will look for a `AWS_ACCESS_KEY_ID` environment variable to pull the value from. (default: {None})
            aws_secret_key {str} -- The secret key of a AWS account with the required permissions. If set to the default `None` keyword argument, we will look for a `AWS_SECRET_ACCESS_KEY` environment variable to pull the value from. (default: {None})
            kms_master_key_id {str} -- The AWS KMS master key ID that will be used to encrypt the archive data. If set to the default `None` keyword argument, you will need to provide a `rsa_key` instead. (default: {None})
            rsa_key {str} -- The RSA key that will be used to encrypt the archive data. A key can be generated through `openssl genrsa -out rubrik_encryption_key.pem 2048`. If set to the default `None` keyword argument, you will need to provide a `kms_master_key_id` instead.  (default: {None})
            archive_name {str} -- The name of the archive location used in the Rubrik GUI. If set to 'default' the following naming convention will be used: "AWS:S3:`aws_bucket_name`" (default: {'default'})
            storage_class {str} -- The AWS storage class you wish to use. (default: {'standard'}) (choices: {standard, 'standard_ia, reduced_redundancy, onezone_ia})
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {180})


        Returns:
            str -- No change required. The '`name`' archival location is already configured on the Rubrik cluster.
            dict -- The full API response for `POST /internal/archive/object_store'`.
        """

        valid_aws_regions = [
            'ap-south-1',
            'ap-northeast-2',
            'ap-southeast-1',
            'ap-southeast-2',
            'ap-northeast-1',
            'ca-central-1',
            'cn-north-1',
            'cn-northwest-1',
            'eu-central-1',
            'eu-west-1',
            'eu-west-2',
            'eu-west-3',
            'sa-east-1',
            'us-gov-west-1',
            'us-west-1',
            'us-east-1',
            'us-east-2',
            'us-west-2']

        valid_storage_classes = [
            'standard',
            'standard_ia',
            'reduced_redundancy',
            'onezone_ia']

        if re.compile(r'[_\/*?%.:|<>]').findall(aws_bucket_name):
            raise InvalidParameterException(
                r"The `aws_bucket_name` may not contain any of the following characters: _\/*?%.:|<>")

        if aws_region is None:
            aws_region = os.environ.get('AWS_DEFAULT_REGION')
            if aws_region is None:
                raise InvalidParameterException("`aws_region` has not been provided.")

        if aws_access_key is None:
            aws_access_key = os.environ.get('AWS_ACCESS_KEY_ID')
            if aws_access_key is None:
                raise InvalidParameterException("`aws_access_key` has not been provided.")

        if aws_secret_key is None:
            aws_secret_key = os.environ.get('AWS_SECRET_ACCESS_KEY')
            if aws_secret_key is None:
                raise InvalidParameterException("`aws_secret_key` has not been provided.")

        if aws_region not in valid_aws_regions:
            raise InvalidParameterException('The `aws_region` must be one of the following: {}'.format(
                valid_aws_regions))

        if storage_class not in valid_storage_classes:
            raise InvalidParameterException('The `storage_class` must be one of the following: {}'.format(
                valid_storage_classes))
        else:
            storage_class = storage_class.upper()

        if archive_name == 'default':
            archive_name = 'AWS:S3:{}'.format(aws_bucket_name.lower().strip())

        if kms_master_key_id is None and rsa_key is None:
            raise InvalidParameterException(
                "You must populated either `kms_master_key_id` or `rsa_key`.")
        elif kms_master_key_id is not None and rsa_key is not None:
            raise InvalidParameterException(
                "Both `kms_master_key_id` or `rsa_key` have been populated. You may only use one.")

        archives_on_cluster = self.get('internal', '/archive/object_store', timeout=timeout)

        config = {}
        config['name'] = archive_name
        config['bucket'] = aws_bucket_name.lower().strip()
        config['defaultRegion'] = aws_region
        config['storageClass'] = storage_class
        config['accessKey'] = aws_access_key
        config['secretKey'] = aws_secret_key
        if kms_master_key_id:
            config['kmsMasterKeyId'] = kms_master_key_id
        elif rsa_key:
            config['pemFileContent'] = rsa_key
        config['objectStoreType'] = 'S3'

        # Create a new dictionary that includes only the values returned by
        # archives_on_cluster
        redacted_archive_definition = {}
        redacted_archive_definition['objectStoreType'] = 'S3'
        redacted_archive_definition['name'] = archive_name
        redacted_archive_definition['accessKey'] = aws_access_key
        redacted_archive_definition['bucket'] = aws_bucket_name.lower().strip()
        redacted_archive_definition['defaultRegion'] = aws_region
        redacted_archive_definition['storageClass'] = storage_class

        for archive in archives_on_cluster['data']:
            # If present, remove the Cloud On Configuration for comparison
            archive_definition = archive['definition']
            for value in ["encryptionType", "defaultComputeNetworkConfig",
                          "isComputeEnabled", "isConsolidationEnabled"]:
                try:
                    del archive_definition[value]
                except BaseException:
                    pass

            if archive_definition == redacted_archive_definition:
                return "No change required. The '{}' archival location is already configured on the Rubrik cluster.".format(
                    archive_name)
            if archive['definition']['objectStoreType'] == 'S3' and archive['definition']['name'] == archive_name:
                raise InvalidParameterException(
                    "Archival location with name '{}' already exists. Please enter a unique `archive_name`.".format(archive_name))

        self.log("aws_s3_cloudout: Creating the AWS S3 archive location.")
        return self.post('internal', '/archive/object_store', config, timeout)

    def update_aws_s3_cloudout(self, current_archive_name, new_archive_name=None, aws_access_key=None, aws_secret_key=None, storage_class=None, timeout=180):  # pylint: ignore
        """Update an AWS S3 archival location on the Rubrik cluster.

        Keyword Arguments:
            current_archive_name {str} -- The name of the current archive to be updated.
            new_archive_name {str} -- Desired name for the updated archive location. If set to default `None` keyword argument, no change will be made. (default: {None})
            aws_access_key {str} -- The access key of a AWS account with the required permissions. If set to the default `None` keyword argument, no change will be made. (default: {None})
            aws_secret_key {str} -- The secret key of a AWS account with the required permissions. If set to the default `None` keyword argument, no change will be made. (default: {None})
            storage_class {str} -- The AWS storage class you wish to use. If set to the default `None` keyword argument, no change will be made. (default: {None}) (choices: {standard, 'standard_ia, reduced_redundancy, onezone_ia})
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {180})


        Returns:
            dict -- The full API response for `PATCH /internal/archive/object_store/{id}'`.
        """

        valid_storage_classes = [
            'standard',
            'standard_ia',
            'reduced_redundancy',
            'onezone_ia']

        if storage_class is not None and storage_class not in valid_storage_classes:
            raise InvalidParameterException(
                'The `storage_class` must be one of the following: {}'.format(valid_storage_classes))

        update_config = None

        self.log("update_aws_s3_cloudout: Searching the Rubrik cluster for S3 archival locations named {}.".format(
            current_archive_name))
        archives_on_cluster = self.get('internal', '/archive/object_store', timeout=timeout)

        for archive in archives_on_cluster['data']:
            # If present, remove the Cloud On Configuration for comparison
            archive_definition = archive['definition']
            try:
                del archive_definition['defaultComputeNetworkConfig']
            except BaseException:
                pass
            if archive['definition']['objectStoreType'] == 'S3' and archive['definition']['name'] == current_archive_name:
                self.log("update_aws_s3_cloudout: Found matching S3 archival location named {}.".format(current_archive_name))
                update_config = archive_definition
                archive_id = archive['id']

        if update_config is None:
            raise InvalidParameterException(
                "No S3 archival location with name '{}' exists.".format(current_archive_name))

        if new_archive_name:
            update_config['name'] = new_archive_name

        if aws_access_key:
            update_config['accessKey'] = aws_access_key

        if aws_secret_key:
            update_config['secretKey'] = aws_secret_key

        if storage_class:
            update_config['storageClass'] = storage_class.upper()

        self.log("update_aws_s3_cloudout: Updating the AWS S3 archive location named {}.".format(current_archive_name))
        return self.patch('internal', '/archive/object_store/{}'.format(archive_id), update_config, timeout)

    def aws_s3_cloudon(self, archive_name, vpc_id, subnet_id, security_group_id, timeout=30):
        """Enable CloudOn for an exsiting AWS S3 archival location.

        Arguments:
            archive_name {str} -- The name of the archive location used in the Rubrik GUI.

        Keyword Arguments:
            vpc_id {str} -- The AWS VPC ID used by Rubrik cluster to launch a temporary Rubrik instance in AWS for instantiation.
            subnet_id {str} -- The AWS Subnet ID used by Rubrik cluster to launch a temporary Rubrik instance in AWS for instantiation.
            security_group_id {str} -- The AWS Security Group ID used by Rubrik cluster to launch a temporary Rubrik instance in AWS for instantiation.
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {30})

        Returns:
            str -- No change required. The '`name`' archival location is already configured on the Rubrik cluster.
            dict -- The full API response for `PATCH /internal/archive/object_store/{id}`.
        """

        self.log("aws_s3_cloudon: Searching the Rubrik cluster for archival locations.")
        archives_on_cluster = self.get('internal', '/archive/object_store', timeout=timeout)

        config = {}
        config['defaultComputeNetworkConfig'] = {}
        config['defaultComputeNetworkConfig']['subnetId'] = subnet_id
        config['defaultComputeNetworkConfig']['vNetId'] = vpc_id
        config['defaultComputeNetworkConfig']['securityGroupId'] = security_group_id

        for archive in archives_on_cluster['data']:
            if archive['definition']['objectStoreType'] == 'S3' and archive['definition']['name'] == archive_name:
                # If present, remove the Cloud On configuration for proper
                # comparison
                try:
                    if archive['definition']['defaultComputeNetworkConfig'] == config['defaultComputeNetworkConfig']:
                        return "No change required. The '{}' archival location is already configured for CloudOn.".format(
                            archive_name)
                except KeyError:
                    self.log("aws_s3_cloudon: Updating the archive location for CloudOn.")
                    return self.patch('internal', "/archive/object_store/{}".format(archive['id']), config, timeout)

        raise InvalidParameterException(
            "The Rubrik cluster does not have an archive location named '{}'.".format(archive_name))

    def azure_cloudout(self, container, azure_access_key, storage_account_name, rsa_key, archive_name='default', instance_type='default', timeout=180):  # pylint: ignore
        """Add a new Azure archival location to the Rubrik cluster.

        Arguments:
            container {str} -- The name of the Azure storage container you wish to use as an archive. The container name will automatically be lowercased and can not contain any of the following characters: `_\/*?%.:\|<>`.
            azure_access_key {str} -- The access key for the Azure storage account.
            storage_account_name {str} -- The name of the Storage Account that the `container` belongs to.
            rsa_key {str} -- The RSA key that will be used to encrypt the archive data. A key can be generated through `openssl genrsa -out rubrik_encryption_key.pem 2048`.

        Keyword Arguments:
            archive_name {str} -- The name of the archive location used in the Rubrik GUI. If set to `default`, the following naming convention will be used: "Azure:`container`" (default: {'default'})
            instance_type {str} -- The Cloud Platform type of the archival location. (default: {'default'}) (choices: {'default', 'china', 'germany', 'government'})
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {180})

        Returns:
            str -- No change required. The '`name`' archival location is already configured on the Rubrik cluster.
            dict -- The full API response for `POST /internal/archive/object_store`.
        """

        container = container.lower()

        if re.compile(r'[_\/*?%.:|<>]').findall(container):
            raise InvalidParameterException(
                r"The `container` may not contain any of the following characters: _\/*?%.:|<>")

        valid_instance_types = ['default', 'china', 'germany', 'government']

        if instance_type not in valid_instance_types:
            raise InvalidParameterException('The `instance_type` argument must be one of the following: {}'.format(
                valid_instance_types))

        if archive_name == 'default':
            archive_name = 'Azure:{}'.format(container)

        self.log("azure_cloudout: Searching the Rubrik cluster for archival locations.")
        archives_on_cluster = self.get('internal', '/archive/object_store', timeout=timeout)

        config = {}
        config['name'] = archive_name
        config['bucket'] = container
        config['accessKey'] = storage_account_name
        config['secretKey'] = azure_access_key
        config['pemFileContent'] = rsa_key
        config['objectStoreType'] = 'Azure'
        if instance_type == 'government':
            config['endpoint'] = 'core.usgovcloudapi.net'
        elif instance_type == 'germany':
            config['endpoint'] = 'core.cloudapi.de'
        elif instance_type == 'china':
            config['endpoint'] = 'core.chinacloudapi.cn'

        # Create a new dictionary that includes only the values returned by
        # archives_on_cluster
        redacted_archive_definition = {}
        redacted_archive_definition['objectStoreType'] = 'Azure'
        redacted_archive_definition['name'] = archive_name
        redacted_archive_definition['accessKey'] = storage_account_name
        redacted_archive_definition['bucket'] = container
        if instance_type == 'government':
            redacted_archive_definition['endpoint'] = 'core.usgovcloudapi.net'
        elif instance_type == 'germany':
            redacted_archive_definition['endpoint'] = 'core.cloudapi.de'
        elif instance_type == 'china':
            redacted_archive_definition['endpoint'] = 'core.chinacloudapi.cn'

        for archive in archives_on_cluster['data']:
            # If present, remove the Cloud On Configuration for comparison
            archive_definition = archive['definition']
            for value in ["encryptionType", "defaultComputeNetworkConfig",
                          "isComputeEnabled", "isConsolidationEnabled", "azureComputeSummary"]:
                try:
                    del archive_definition[value]
                except BaseException:
                    pass

            if archive_definition == redacted_archive_definition:
                return "No change required. The '{}' archival location is already configured on the Rubrik cluster.".format(
                    archive_name)

            if archive_definition['objectStoreType'] == 'Azure' and archive_definition['name'] == archive_name:
                raise InvalidParameterException("Archival location with name '{}' already exists. Please enter a unique `name`.".format(
                    archive_name))

        self.log("azure_cloudout: Creating the Azure archive location.")
        return self.post('internal', '/archive/object_store', config)

    def azure_cloudon(self, archive_name, container, storage_account_name, application_id, application_key, tenant_id, region, virtual_network_id, subnet_name, security_group_id, timeout=30):  # pylint: ignore
        """Enable CloudOn for an exsiting AWS S3 archival location.

        Arguments:
            archive_name {str} -- The name of the archive location used in the Rubrik GUI.
            container {str} -- The name of the Azure storage container being used as the archive target. The container name will automatically be lowercased and can not contain any of the following characters: `_\/*?%.:\|<>`.
            storage_account_name {str} -- The name of the Storage Account that the `container` belongs to.
            application_id {str} -- The ID of the application registered in Azure Active Directory.
            application_key {str} -- The key of the application registered in Azure Active Directory.
            tenant_id {str} -- The tenant ID, also known as the directory ID, found under the Azure Active Directory properties.
            region {str} -- The name of the Azure region where the `container` is located. (choices: {westus, westus2, centralus, eastus, eastus2, northcentralus, southcentralus, westcentralus, canadacentral, canadaeast, brazilsouth, northeurope, westeurope, uksouth, ukwest, eastasia, southeastasia, japaneast, japanwest, australiaeast australiasoutheast, centralindia, southindia, westindia, koreacentral, koreasouth})
            virtual_network_id {str} -- The Azure virtual network ID used by Rubrik cluster to launch a temporary Rubrik instance in Azure for instantiation.
            subnet_name {str} -- The Azure subnet name used by Rubrik cluster to launch a temporary Rubrik instance in Azure for instantiation.
            security_group_id {str} -- The Azure Security Group ID used by Rubrik cluster to launch a temporary Rubrik instance in Azure for instantiation.

        Keyword Arguments:
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {30})


        Returns:
            str -- No change required. The '`archive_name`' archival location is already configured for CloudOn.
            dict -- The full API response for `PATCH /internal/archive/object_store/{id}`.
        """

        valid_regions = [
            "westus",
            "westus2",
            "centralus",
            "eastus",
            "eastus2",
            "northcentralus",
            "southcentralus",
            "westcentralus",
            "canadacentral",
            "canadaeast",
            "brazilsouth",
            "northeurope",
            "westeurope",
            "uksouth",
            "ukwest",
            "eastasia",
            "southeastasia",
            "japaneast",
            "japanwest",
            "australiaeast",
            "australiasoutheast",
            "centralindia",
            "southindia",
            "westindia",
            "koreacentral",
            "koreasouth"]

        if region not in valid_regions:
            raise InvalidParameterException('The `region` must be one of the following: {}'.format(valid_regions))

        self.log("azure_cloudon: Searching the Rubrik cluster for archival locations.")
        archives_on_cluster = self.get('internal', '/archive/object_store', timeout=timeout)

        config = {}
        config['name'] = archive_name
        config['objectStoreType'] = 'Azure'
        config['isComputeEnabled'] = True

        config['azureComputeSummary'] = {}
        config['azureComputeSummary']["tenantId"] = tenant_id
        config['azureComputeSummary']["subscriptionId"] = virtual_network_id.split("/")[2]
        config['azureComputeSummary']["clientId"] = application_id
        config['azureComputeSummary']["region"] = region
        config['azureComputeSummary']["generalPurposeStorageAccountName"] = storage_account_name
        config['azureComputeSummary']["containerName"] = container

        config['azureComputeSecret'] = {}
        config['azureComputeSecret']["clientSecret"] = application_key
        config['defaultComputeNetworkConfig'] = {}
        config['defaultComputeNetworkConfig']['subnetId'] = subnet_name
        config['defaultComputeNetworkConfig']['vNetId'] = virtual_network_id
        config['defaultComputeNetworkConfig']['securityGroupId'] = security_group_id

        redacted_archive_definition = {}
        redacted_archive_definition['name'] = archive_name
        redacted_archive_definition['objectStoreType'] = "Azure"
        redacted_archive_definition['accessKey'] = storage_account_name
        redacted_archive_definition['bucket'] = container
        redacted_archive_definition['isComputeEnabled'] = True

        redacted_archive_definition['azureComputeSummary'] = {}
        redacted_archive_definition['azureComputeSummary']["tenantId"] = tenant_id
        redacted_archive_definition['azureComputeSummary']["subscriptionId"] = virtual_network_id.split("/")[2]
        redacted_archive_definition['azureComputeSummary']["clientId"] = application_id
        redacted_archive_definition['azureComputeSummary']["region"] = region
        redacted_archive_definition['azureComputeSummary']["generalPurposeStorageAccountName"] = storage_account_name
        redacted_archive_definition['azureComputeSummary']["containerName"] = container

        redacted_archive_definition['defaultComputeNetworkConfig'] = {}
        redacted_archive_definition['defaultComputeNetworkConfig']['subnetId'] = subnet_name
        redacted_archive_definition['defaultComputeNetworkConfig']['vNetId'] = virtual_network_id
        redacted_archive_definition['defaultComputeNetworkConfig']['securityGroupId'] = security_group_id

        for archive in archives_on_cluster['data']:

            archive_definition = archive['definition']
            for value in ["isConsolidationEnabled", "encryptionType"]:
                try:
                    del archive_definition[value]
                except BaseException:
                    pass

            try:
                del archive_definition["azureComputeSummary"]["environment"]
            except BaseException:
                pass

            try:
                del archive_definition["defaultComputeNetworkConfig"]["resourceGroupId"]
            except BaseException:
                pass

            if archive['definition']['objectStoreType'] == 'Azure' and archive['definition']['name'] == archive_name:

                if archive['definition'] == redacted_archive_definition:

                    return "No change required. The '{}' archival location is already configured for CloudOn.".format(
                        archive_name)
                else:
                    self.log("azure_cloudon: Updating the archive location for CloudOn.")
                    return self.patch('internal', "/archive/object_store/{}".format(archive['id']), config, timeout)

        raise InvalidParameterException(
            "The Rubrik cluster does not have an archive location named '{}'.".format(archive_name))

    def add_aws_native_account(self, aws_account_name, aws_access_key=None, aws_secret_key=None, aws_regions=None, regional_bolt_network_configs=None, timeout=30):  # pylint: ignore
        """Add a new AWS account to EC2 native protection on the Rubrik cluster.

        Arguments:
            aws_account_name {str} -- The name of the AWS account you wish to protect. This is the name that will be displayed in the Rubrik UI.

        Keyword Arguments:
            aws_access_key {str} -- The access key of a AWS account with the required permissions. If set to the default `None` keyword argument, we will look for a `AWS_ACCESS_KEY_ID` environment variable to pull the value from. (default: {None})
            aws_secret_key {str} -- The secret key of a AWS account with the required permissions. If set to the default `None` keyword argument, we will look for a `AWS_SECRET_ACCESS_KEY` environment variable to pull the value from. (default: {None})
            aws_regions {list} -- List of AWS regions to protect in this AWS account. If set to the default `None` keyword argument, we will look for a `AWS_DEFAULT_REGION` environment variable to pull the value from. (default: {None}) (choices: {ap-south-1, ap-northeast-3, ap-northeast-2, ap-southeast-1, ap-southeast-2, ap-northeast-1, ca-central-1, cn-north-1, cn-northwest-1, eu-central-1, eu-west-1, eu-west-2, eu-west-3, us-west-1, us-east-1, us-east-2, us-west-2})
            regional_bolt_network_configs {list of dicts} -- List of dicts containing per region bolt network configs. (ex. dict format: {"region": "aws-region-name", "vNetId": "aws-vpc-id", "subnetId": "aws-subnet-id", "securityGroupId": "aws-subnet-id"}) (default: {None})
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {30})


        Returns:
            str -- No change required. Cloud native source with access key `aws_access_key` is already configured on the Rubrik cluster.
            dict -- The full API response for `POST /internal/aws/account'`.
        """

        valid_aws_regions = [
            'ap-south-1',
            'ap-northeast-3',
            'ap-northeast-2',
            'ap-southeast-1',
            'ap-southeast-2',
            'ap-northeast-1',
            'ca-central-1',
            'cn-north-1',
            'cn-northwest-1',
            'eu-central-1',
            'eu-west-1',
            'eu-west-2',
            'eu-west-3',
            'us-west-1',
            'us-east-1',
            'us-east-2',
            'us-west-2']

        # verify we are on cdm 4.2 or newer, required for cloud native
        # protection
        if self.minimum_installed_cdm_version(4.2) is False:
            raise CDMVersionException(4.2)

        # check for regions and credentials in environment variables if not
        # provided explicitly
        if aws_regions is None:
            aws_regions = [os.environ.get('AWS_DEFAULT_REGION')]
            if aws_regions == [None]:
                raise InvalidParameterException("`aws_region` has not been provided.")

        if aws_access_key is None:
            aws_access_key = os.environ.get('AWS_ACCESS_KEY_ID')
            if aws_access_key is None:
                raise InvalidParameterException("`aws_access_key` has not been provided.")

        if aws_secret_key is None:
            aws_secret_key = os.environ.get('AWS_SECRET_ACCESS_KEY')
            if aws_secret_key is None:
                raise InvalidParameterException("`aws_secret_key` has not been provided.")

        # verify supplied regions are in the supported list of regions for
        # cloud native protection
        if any(aws_region not in valid_aws_regions for aws_region in aws_regions):
            raise InvalidParameterException(
                'The list `aws_regions` may only contain the following values: {}'.format(valid_aws_regions))

        # verify that our regional_bolt_network_configs are either None or in a
        # list
        if isinstance(regional_bolt_network_configs, list) is False and regional_bolt_network_configs is not None:
            raise InvalidTypeException("`regional_bolt_network_configs` must be a list if defined.")

        if regional_bolt_network_configs is not None:

            # verify our list of bolt_network_configs only contains dicts
            for bolt_network_config in regional_bolt_network_configs:
                if isinstance(bolt_network_config, dict) is False:
                    raise InvalidTypeException("The `regional_bolt_network_configs` list can only contain dicts.")

                # verify that all the required paramteters are provided in all
                # regional_bolt_network_configs
                if any(requiredkey not in bolt_network_config for requiredkey in [
                        'region',
                        'vNetId',
                        'subnetId',
                        'securityGroupId']):
                    raise InvalidParameterException(
                        "Each `regional_bolt_network_config` dict must contain the following keys: 'region', 'vNetId', 'subnetId', 'securityGroupId'.")

        self.log("aws_native_account: Searching the Rubrik cluster for cloud native sources.")
        cloud_native_on_cluster = self.get('internal', '/aws/account', timeout=timeout)

        for cloud_source in cloud_native_on_cluster['data']:

            # verify a cloud native source with this name does not exist
            # already
            self.log("aws_native_account: Validating no conflict with `{}`".format(cloud_source['id']))
            if cloud_source['name'] == aws_account_name:
                raise InvalidParameterException("Cloud native source with name '{}' already exists. Please enter a unique `aws_account_name`.".format(
                    aws_account_name))

            # idempotent return if a cloud native source with this access key
            # already exists
            cloud_source_detail = self.get('internal', '/aws/account/{}'.format(cloud_source['id']), timeout=timeout)
            if cloud_source_detail['accessKey'] == aws_access_key:
                return "No change required. Cloud native source with access key '{}' is already configured on the Rubrik cluster.".format(
                    aws_access_key)

        # build the config for our API call
        config = {}
        config['name'] = aws_account_name
        config['accessKey'] = aws_access_key
        config['secretKey'] = aws_secret_key
        config['regions'] = aws_regions

        # only include bolt configs if they were supplied
        if regional_bolt_network_configs is not None:
            config['regionalBoltNetworkConfigs'] = regional_bolt_network_configs

        # make the API call, return the result
        self.log("aws_native_account: Creating the cloud native source.")
        return self.post('internal', '/aws/account', config, timeout)

    def update_aws_native_account(self, aws_account_name, config, timeout=15):
        """Update an exsiting AWS account used for EC2 native protection on the Rubrik cluster.

        Arguments:
            aws_account_name {str} -- The name of the AWS account you wish to update. This is the name that is displayed in the Rubrik UI.

        Keyword Arguments:
            config {dict} -- The configuration to use to update the AWS account. Full example values can be found in the Rubrik API Playground for the PATCH /aws/account/{id} endpoint
            timeout {int} -- The number of seconds to wait to establish a connection the Rubrik cluster before returning a timeout error. (default: {15})


        Returns:
            dict -- The full API response for `PATCH /aws/account/{id}`.
        """
        # verify we are on cdm 4.2 or newer, required for cloud native
        # protection
        if self.minimum_installed_cdm_version(4.2) is False:
            raise CDMVersionException(4.2)

        if not isinstance(config, dict):
            raise InvalidTypeException("The 'config' argument must be a dictionary.")

        self.log("update_aws_native_account: Checking the Rubrik cluster for the AWS Native Account.")
        account_id = self.object_id(aws_account_name, "aws_native", timeout=timeout)

        self.log("update_aws_native_account: Updating the AWS Native Account.")
        return self.patch("internal", "/aws/account/{}".format(account_id), config)
